@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2006-2009 Stig Venaas <venaas@uninett.no>
- * Copyright (C) 2010, 2011 NORDUnet A/S
+ * Copyright (C) 2010,2011,2012 NORDUnet A/S
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -64,6 +64,7 @@
 #include <libgen.h>
 #include <pthread.h>
 #include <errno.h>
+#include <assert.h>
 #include <openssl/ssl.h>
 #include <openssl/rand.h>
 #include <openssl/err.h>
@@ -326,8 +327,13 @@ void freeserver(struct server *server, uint8_t destroymutex) {
     if (server->rbios)
 	freebios(server->rbios);
     free(server->dynamiclookuparg);
-    if (server->ssl)
-	SSL_free(server->ssl);
+    if (server->ssl) {
+#if defined ENABLE_EXPERIMENTAL_DYNDISC
+        if (server->sock >= 0)
+            close(server->sock);
+#endif
+        SSL_free(server->ssl);
+    }
     if (destroymutex) {
 	pthread_mutex_destroy(&server->lock);
 	pthread_cond_destroy(&server->newrq_cond);
@@ -767,7 +773,12 @@ int hasdynamicserver(struct list *srvconfs) {
     struct list_node *entry;
 
     for (entry = list_first(srvconfs); entry; entry = list_next(entry))
-	if (((struct clsrvconf *)entry->data)->dynamiclookupcommand)
+#if defined ENABLE_EXPERIMENTAL_DYNDISC
+        if (((struct clsrvconf *)entry->data)->dynamiclookupcommand
+            || ((struct clsrvconf *)entry->data)->servers->in_use)
+#else
+        if (((struct clsrvconf *)entry->data)->dynamiclookupcommand)
+#endif
 	    return 1;
     return 0;
 }
@@ -1306,6 +1317,8 @@ struct clsrvconf *choosesrvconf(struct list *srvconfs) {
 	server = (struct clsrvconf *)entry->data;
 	if (!server->servers)
 	    return server;
+        if (server->servers->dynfailing)
+            continue;
 	if (!first)
 	    first = server;
 	if (!server->servers->connectionok && !server->servers->dynstartup)
@@ -1344,11 +1357,15 @@ struct server *findserver(struct realm **realm, struct tlv *username, uint8_t ac
 	    pthread_mutex_unlock(&(*realm)->mutex);
 	    freerealm(*realm);
 	    *realm = subrealm;
+            debug(DBG_DBG, "added realm: %s", (*realm)->name);
 	    srvconf = choosesrvconf(acc ? (*realm)->accsrvconfs : (*realm)->srvconfs);
+            debug(DBG_DBG, "found conf for new realm: %s", srvconf->name);
 	}
     }
-    if (srvconf)
+    if (srvconf) {
+        debug(DBG_DBG, "found matching conf: %s", srvconf->name);
 	server = srvconf->servers;
+    }
 
 exit:
     free(id);
@@ -1751,18 +1768,30 @@ void *clientwr(void *arg) {
 
     conf = server->conf;
 
+#define ZZZ 900
+
     if (server->dynamiclookuparg && !dynamicconfig(server)) {
 	dynconffail = 1;
 	server->dynstartup = 0;
-	sleep(900);
+	server->dynfailing = 1;
+#if defined ENABLE_EXPERIMENTAL_DYNDISC
+	pthread_mutex_unlock(&server->lock);
+#endif
+	debug(DBG_WARN, "%s: dynamicconfig(%s) failed, sleeping %ds",
+              __func__, server->conf->name, ZZZ);
+	sleep(ZZZ);
 	goto errexit;
     }
-
-    if (!resolvehostports(conf->hostports, conf->pdef->socktype)) {
-	debug(DBG_WARN, "clientwr: resolve failed");
-	server->dynstartup = 0;
-	sleep(900);
-	goto errexit;
+#if defined ENABLE_EXPERIMENTAL_DYNDISC
+    pthread_mutex_unlock(&server->lock);
+#endif
+    /* FIXME: Is resolving not always done by compileserverconfig(),
+     * either as part of static configuration setup or by
+     * dynamicconfig() above?  */
+    if (!resolvehostports(conf->hostports, conf->hostaf, conf->pdef->socktype)) {
+        debug(DBG_WARN, "%s: resolve failed, sleeping %ds", __func__, ZZZ);
+        sleep(ZZZ);
+        goto errexit;
     }
 
     memset(&timeout, 0, sizeof(struct timespec));
@@ -1775,12 +1804,18 @@ void *clientwr(void *arg) {
     if (conf->pdef->connecter) {
 	if (!conf->pdef->connecter(server, NULL, server->dynamiclookuparg ? 5 : 0, "clientwr")) {
 	    if (server->dynamiclookuparg) {
-		server->dynstartup = 0;
-		sleep(900);
+                server->dynstartup = 0;
+		server->dynfailing = 1;
+                debug(DBG_WARN, "%s: connect failed, sleeping %ds",
+                      __func__, ZZZ);
+		sleep(ZZZ);
 	    }
 	    goto errexit;
 	}
 	server->connectionok = 1;
+#if defined ENABLE_EXPERIMENTAL_DYNDISC
+	server->in_use = 1;
+#endif
 	if (pthread_create(&clientrdth, NULL, conf->pdef->clientconnreader, (void *)server)) {
 	    debugerrno(errno, DBG_ERR, "clientwr: pthread_create failed");
 	    goto errexit;
@@ -1890,6 +1925,9 @@ void *clientwr(void *arg) {
 	}
     }
 errexit:
+#if defined ENABLE_EXPERIMENTAL_DYNDISC
+    server->in_use = 0;
+#endif
     conf->servers = NULL;
     if (server->dynamiclookuparg) {
 	removeserversubrealms(realms, conf);
@@ -1909,7 +1947,7 @@ void createlistener(uint8_t type, char *arg) {
     int s = -1, on = 1, *sp = NULL;
     struct hostportres *hp = newhostport(arg, protodefs[type]->portdefault, 0);
 
-    if (!hp || !resolvehostport(hp, protodefs[type]->socktype, 1))
+    if (!hp || !resolvehostport(hp, AF_UNSPEC, protodefs[type]->socktype, 1))
 	debugx(1, DBG_ERR, "createlistener: failed to resolve %s", arg);
 
     for (res = hp->addrinfo; res; res = res->ai_next) {
@@ -2173,16 +2211,40 @@ struct list *createsubrealmservers(struct realm *realm, struct list *srvconfs) {
 		debug(DBG_ERR, "malloc failed");
 		continue;
 	    }
+            debug(DBG_DBG, "%s: copying config %s", __func__, conf->name);
 	    *srvconf = *conf;
+            /* Shallow copy -- sharing all the pointers.  addserver()
+             * will take care of servers (which btw has to be NUL) but
+             * the rest of them are shared with the config found in
+             * the srvconfs list.  */
 	    if (addserver(srvconf)) {
 		srvconf->servers->dynamiclookuparg = stringcopy(realm->name, 0);
 		srvconf->servers->dynstartup = 1;
+                debug(DBG_DBG, "%s: new client writer for %s",
+                      __func__, srvconf->servers->conf->name);
+#if defined ENABLE_EXPERIMENTAL_DYNDISC
+        	pthread_mutex_lock(&srvconf->servers->lock);
+#endif
 		if (pthread_create(&clientth, NULL, clientwr, (void *)(srvconf->servers))) {
+#if defined ENABLE_EXPERIMENTAL_DYNDISC
+                    pthread_mutex_unlock(&srvconf->servers->lock);
+#endif
 		    debugerrno(errno, DBG_ERR, "pthread_create failed");
 		    freeserver(srvconf->servers, 1);
 		    srvconf->servers = NULL;
+#if defined ENABLE_EXPERIMENTAL_DYNDISC
+		    conf = srvconf;
+		    continue;
+#endif
 		} else
 		    pthread_detach(clientth);
+
+#if defined ENABLE_EXPERIMENTAL_DYNDISC
+                /* If clientwr() could not find a NAPTR we have to
+                 * wait for dynfailing=1 what is set in clientwr().  */
+                pthread_mutex_lock(&srvconf->servers->lock);
+                pthread_mutex_unlock(&srvconf->servers->lock);
+#endif
 	    }
 	    conf = srvconf;
 	}
@@ -2273,14 +2335,9 @@ int dynamicconfig(struct server *server) {
     }
 
     if (status) {
-	if (WEXITSTATUS(status) == 10) {
-	    debug(DBG_INFO, "dynamicconfig: command signals empty config");
-	}
-	else {
-	    debug(DBG_INFO, "dynamicconfig: command exited with status %d",
-		  WEXITSTATUS(status));
-	    goto errexit;
-	}
+        debug(DBG_INFO, "dynamicconfig: command exited with status %d",
+              WEXITSTATUS(status));
+        goto errexit;
     }
 
     if (ok)
@@ -2544,6 +2601,9 @@ int setttlattr(struct options *opts, char *defaultattr) {
 }
 
 void freeclsrvconf(struct clsrvconf *conf) {
+    assert(conf);
+    assert(conf->name);
+    debug(DBG_DBG, "%s: freeing %p (%s)", __func__, conf, conf->name);
     free(conf->name);
     if (conf->hostsrc)
 	freegconfmstr(conf->hostsrc);
@@ -2646,7 +2706,10 @@ int mergesrvconf(struct clsrvconf *dst, struct clsrvconf *src) {
 	!mergeconfstring(&dst->matchcertattr, &src->matchcertattr) ||
 	!mergeconfstring(&dst->confrewritein, &src->confrewritein) ||
 	!mergeconfstring(&dst->confrewriteout, &src->confrewriteout) ||
-	!mergeconfstring(&dst->dynamiclookupcommand, &src->dynamiclookupcommand))
+	!mergeconfstring(&dst->confrewriteusername, &src->confrewriteusername) ||
+	!mergeconfstring(&dst->dynamiclookupcommand, &src->dynamiclookupcommand) ||
+	!mergeconfstring(&dst->fticks_viscountry, &src->fticks_viscountry) ||
+	!mergeconfstring(&dst->fticks_visinst, &src->fticks_visinst))
 	return 0;
     if (src->pdef)
 	dst->pdef = src->pdef;
@@ -2659,10 +2722,31 @@ int mergesrvconf(struct clsrvconf *dst, struct clsrvconf *src) {
     return 1;
 }
 
+/** Set *AF according to IPV4ONLY and IPV6ONLY:
+    - If both are set, the function fails.
+    - If exactly one is set, *AF is set accordingly.
+    - If none is set, *AF is not affected.
+    Return 0 on success and !0 on failure.
+    In the case of an error, *AF is not affected.  */
+int config_hostaf(const char *desc, int ipv4only, int ipv6only, int *af) {
+    assert(af != NULL);
+    if (ipv4only && ipv6only) {
+	debug(DBG_ERR, "error in block %s, at most one of IPv4Only and "
+              "IPv6Only can be enabled", desc);
+        return -1;
+    }
+    if (ipv4only)
+        *af = AF_INET;
+    if (ipv6only)
+        *af = AF_INET6;
+    return 0;
+}
+
 int confclient_cb(struct gconffile **cf, void *arg, char *block, char *opt, char *val) {
     struct clsrvconf *conf;
     char *conftype = NULL, *rewriteinalias = NULL;
     long int dupinterval = LONG_MIN, addttl = LONG_MIN;
+    uint8_t ipv4only = 0, ipv6only = 0;
 
     debug(DBG_DBG, "confclient_cb called for %s", block);
 
@@ -2676,6 +2760,8 @@ int confclient_cb(struct gconffile **cf, void *arg, char *block, char *opt, char
 	    cf, block,
 	    "type", CONF_STR, &conftype,
 	    "host", CONF_MSTR, &conf->hostsrc,
+            "IPv4Only", CONF_BLN, &ipv4only,
+            "IPv6Only", CONF_BLN, &ipv6only,
 	    "secret", CONF_STR, &conf->secret,
 #if defined(RADPROT_TLS) || defined(RADPROT_DTLS)
 	    "tls", CONF_STR, &conf->tls,
@@ -2690,6 +2776,7 @@ int confclient_cb(struct gconffile **cf, void *arg, char *block, char *opt, char
 	    "rewriteattribute", CONF_STR, &conf->confrewriteusername,
 #if defined(WANT_FTICKS)
 	    "fticksVISCOUNTRY", CONF_STR, &conf->fticks_viscountry,
+	    "fticksVISINST", CONF_STR, &conf->fticks_visinst,
 #endif
 	    NULL
 	    ))
@@ -2716,7 +2803,9 @@ int confclient_cb(struct gconffile **cf, void *arg, char *block, char *opt, char
 
 #if defined(RADPROT_TLS) || defined(RADPROT_DTLS)
     if (conf->type == RAD_TLS || conf->type == RAD_DTLS) {
-	conf->tlsconf = conf->tls ? tlsgettls(conf->tls, NULL) : tlsgettls("defaultclient", "default");
+	conf->tlsconf = conf->tls
+            ? tlsgettls(conf->tls, NULL)
+            : tlsgettls("defaultClient", "default");
 	if (!conf->tlsconf)
 	    debugx(1, DBG_ERR, "error in block %s, no tls context defined", block);
 	if (conf->matchcertattr && !addmatchcertattr(conf))
@@ -2724,6 +2813,12 @@ int confclient_cb(struct gconffile **cf, void *arg, char *block, char *opt, char
     }
 #endif
 
+    conf->hostaf = AF_UNSPEC;
+    if (config_hostaf("top level", options.ipv4only, options.ipv6only, &conf->hostaf))
+        debugx(1, DBG_ERR, "config error: ^");
+    if (config_hostaf(block, ipv4only, ipv6only, &conf->hostaf))
+        debugx(1, DBG_ERR, "error in block %s: ^", block);
+
     if (dupinterval != LONG_MIN) {
 	if (dupinterval < 0 || dupinterval > 255)
 	    debugx(1, DBG_ERR, "error in block %s, value of option DuplicateInterval is %d, must be 0-255", block, dupinterval);
@@ -2741,7 +2836,9 @@ int confclient_cb(struct gconffile **cf, void *arg, char *block, char *opt, char
 	conf->confrewritein = rewriteinalias;
     else
 	free(rewriteinalias);
-    conf->rewritein = conf->confrewritein ? getrewrite(conf->confrewritein, NULL) : getrewrite("defaultclient", "default");
+    conf->rewritein = conf->confrewritein
+        ? getrewrite(conf->confrewritein, NULL)
+        : getrewrite("defaultClient", "default");
     if (conf->confrewriteout)
 	conf->rewriteout = getrewrite(conf->confrewriteout, NULL);
 
@@ -2752,8 +2849,8 @@ int confclient_cb(struct gconffile **cf, void *arg, char *block, char *opt, char
     }
 
     if (!addhostport(&conf->hostports, conf->hostsrc, conf->pdef->portdefault, 1) ||
-	!resolvehostports(conf->hostports, conf->pdef->socktype))
-	debugx(1, DBG_ERR, "resolve failed, exiting");
+	!resolvehostports(conf->hostports, conf->hostaf, conf->pdef->socktype))
+	debugx(1, DBG_ERR, "%s: resolve failed, exiting", __func__);
 
     if (!conf->secret) {
 	if (!conf->pdef->secretdefault)
@@ -2776,7 +2873,9 @@ int confclient_cb(struct gconffile **cf, void *arg, char *block, char *opt, char
 int compileserverconfig(struct clsrvconf *conf, const char *block) {
 #if defined(RADPROT_TLS) || defined(RADPROT_DTLS)
     if (conf->type == RAD_TLS || conf->type == RAD_DTLS) {
-    	conf->tlsconf = conf->tls ? tlsgettls(conf->tls, NULL) : tlsgettls("defaultserver", "default");
+    	conf->tlsconf = conf->tls
+            ? tlsgettls(conf->tls, NULL)
+            : tlsgettls("defaultServer", "default");
 	if (!conf->tlsconf) {
 	    debug(DBG_ERR, "error in block %s, no tls context defined", block);
 	    return 0;
@@ -2801,7 +2900,9 @@ int compileserverconfig(struct clsrvconf *conf, const char *block) {
     if (conf->retrycount == 255)
 	conf->retrycount = conf->pdef->retrycountdefault;
 
-    conf->rewritein = conf->confrewritein ? getrewrite(conf->confrewritein, NULL) : getrewrite("defaultserver", "default");
+    conf->rewritein = conf->confrewritein
+        ? getrewrite(conf->confrewritein, NULL)
+        : getrewrite("defaultServer", "default");
     if (conf->confrewriteout)
 	conf->rewriteout = getrewrite(conf->confrewriteout, NULL);
 
@@ -2810,8 +2911,10 @@ int compileserverconfig(struct clsrvconf *conf, const char *block) {
 	return 0;
     }
 
-    if (!conf->dynamiclookupcommand && !resolvehostports(conf->hostports, conf->pdef->socktype)) {
-	debug(DBG_ERR, "resolve failed, exiting");
+    if (!conf->dynamiclookupcommand &&
+        !resolvehostports(conf->hostports, conf->hostaf,
+                          conf->pdef->socktype)) {
+	debug(DBG_ERR, "%s: resolve failed", __func__);
 	return 0;
     }
     return 1;
@@ -2821,6 +2924,7 @@ int confserver_cb(struct gconffile **cf, void *arg, char *block, char *opt, char
     struct clsrvconf *conf, *resconf;
     char *conftype = NULL, *rewriteinalias = NULL;
     long int retryinterval = LONG_MIN, retrycount = LONG_MIN, addttl = LONG_MIN;
+    uint8_t ipv4only = 0, ipv6only = 0;
 
     debug(DBG_DBG, "confserver_cb called for %s", block);
 
@@ -2841,6 +2945,8 @@ int confserver_cb(struct gconffile **cf, void *arg, char *block, char *opt, char
     if (!getgenericconfig(cf, block,
 			  "type", CONF_STR, &conftype,
 			  "host", CONF_MSTR, &conf->hostsrc,
+                          "IPv4Only", CONF_BLN, &ipv4only,
+                          "IPv6Only", CONF_BLN, &ipv6only,
 			  "port", CONF_STR, &conf->portsrc,
 			  "secret", CONF_STR, &conf->secret,
 #if defined(RADPROT_TLS) || defined(RADPROT_DTLS)
@@ -2888,6 +2994,12 @@ int confserver_cb(struct gconffile **cf, void *arg, char *block, char *opt, char
     free(conftype);
     conftype = NULL;
 
+    conf->hostaf = AF_UNSPEC;
+    if (config_hostaf("top level", options.ipv4only, options.ipv6only, &conf->hostaf))
+        debugx(1, DBG_ERR, "config error: ^");
+    if (config_hostaf(block, ipv4only, ipv6only, &conf->hostaf))
+        goto errexit;
+
     conf->pdef = protodefs[conf->type];
 
     if (!conf->confrewritein)
@@ -2935,7 +3047,8 @@ int confserver_cb(struct gconffile **cf, void *arg, char *block, char *opt, char
 
     if (resconf || !conf->dynamiclookupcommand) {
 	if (!compileserverconfig(conf, block))
-	    goto errexit;
+            return 0; /* Don't goto errexit and free resconf -- it's
+                       * not ours to free.  */
     }
 
     if (!conf->secret) {
@@ -3069,6 +3182,7 @@ void getmainconfig(const char *configfile) {
 	    "ListenDTLS", CONF_MSTR, &listenargs[RAD_DTLS],
 	    "SourceDTLS", CONF_STR, &sourcearg[RAD_DTLS],
 #endif
+            "PidFile", CONF_STR, &options.pidfile,
 	    "TTLAttribute", CONF_STR, &options.ttlattr,
 	    "addTTL", CONF_LINT, &addttl,
 	    "LogLevel", CONF_LINT, &loglevel,
@@ -3085,7 +3199,10 @@ void getmainconfig(const char *configfile) {
 	    "FTicksReporting", CONF_STR, &fticks_reporting_str,
 	    "FTicksMAC", CONF_STR, &fticks_mac_str,
 	    "FTicksKey", CONF_STR, &fticks_key_str,
+	    "FTicksSyslogFacility", CONF_STR, &options.ftickssyslogfacility,
 #endif
+            "IPv4Only", CONF_BLN, &options.ipv4only,
+            "IPv6Only", CONF_BLN, &options.ipv6only,
 	    NULL
 	    ))
 	debugx(1, DBG_ERR, "configuration error");
@@ -3239,8 +3356,18 @@ int radsecproxy_main(int argc, char **argv) {
 	options.loglevel = loglevel;
     else if (options.loglevel)
 	debug_set_level(options.loglevel);
-    if (!foreground)
-	debug_set_destination(options.logdestination ? options.logdestination : "x-syslog:///");
+    if (!foreground) {
+	debug_set_destination(options.logdestination
+                              ? options.logdestination
+                              : "x-syslog:///", LOG_TYPE_DEBUG);
+#if defined(WANT_FTICKS)
+    	if (options.ftickssyslogfacility) {
+            debug_set_destination(options.ftickssyslogfacility,
+                                  LOG_TYPE_FTICKS);
+            free(options.ftickssyslogfacility);
+    	}
+#endif
+    }
     free(options.logdestination);
 
     if (!list_first(clconfs))
@@ -3256,6 +3383,8 @@ int radsecproxy_main(int argc, char **argv) {
 
     debug_timestamp_on();
     debug(DBG_INFO, "radsecproxy revision %s starting", PACKAGE_VERSION);
+    if (!pidfile)
+        pidfile = options.pidfile;
     if (pidfile && !createpidfile(pidfile))
 	debugx(1, DBG_ERR, "failed to create pidfile %s: %s", pidfile, strerror(errno));
 
