@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2006-2009 Stig Venaas <venaas@uninett.no>
- * Copyright (C) 2010,2011 NORDUnet A/S
+ * Copyright (C) 2010,2011,2012 NORDUnet A/S
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -18,7 +18,7 @@
 
 #define DEBUG_LEVEL 2
 
-#define CONFIG_MAIN "/etc/radsecproxy.conf"
+#define CONFIG_MAIN SYSCONFDIR"/radsecproxy.conf"
 
 /* MAX_REQUESTS must be 256 due to Radius' 8 bit ID field */
 #define MAX_REQUESTS 256
@@ -54,7 +54,9 @@ enum rsp_fticks_mac_type {
 };
 
 struct options {
+    char *pidfile;
     char *logdestination;
+    char *ftickssyslogfacility;
     char *ttlattr;
     uint32_t ttlattrtype[2];
     uint8_t addttl;
@@ -63,6 +65,8 @@ struct options {
     enum rsp_fticks_reporting_type fticks_reporting;
     enum rsp_fticks_mac_type fticks_mac;
     uint8_t *fticks_key;
+    uint8_t ipv4only;
+    uint8_t ipv6only;
 };
 
 struct commonprotoopts {
@@ -104,6 +108,7 @@ struct clsrvconf {
     uint8_t type; /* RAD_UDP/RAD_TLS/RAD_TCP */
     const struct protodefs *pdef;
     char **hostsrc;
+    int hostaf;
     char *portsrc;
     struct list *hostports;
     char *secret;
@@ -130,6 +135,7 @@ struct clsrvconf {
     struct list *clients;
     struct server *servers;
     char *fticks_viscountry;
+    char *fticks_visinst;
 };
 
 #include "tlscommon.h"
@@ -157,6 +163,10 @@ struct server {
     uint8_t connectionok;
     uint8_t lostrqs;
     uint8_t dynstartup;
+    uint8_t dynfailing;
+#if defined ENABLE_EXPERIMENTAL_DYNDISC
+    uint8_t in_use;
+#endif
     char *dynamiclookuparg;
     int nextid;
     struct timeval lastrcv;
